@@ -16,8 +16,7 @@
 # Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+# this program. If not, see <https://www.gnu.org/licenses/>
 
 
 """Analyze failed piuparts logs and move them around when the errors are known.
@@ -34,18 +33,124 @@ move the failed log to ./bugged as well.
 
 import os
 import sys
+import time
 import re
 import shutil
 import subprocess
+import fcntl
 
 import debianbts
 import apt_pkg
+from signal import alarm, signal, SIGALRM
+from collections import deque
+
+import piupartslib.conf
+from piupartslib.conf import MissingSection
+
+
+CONFIG_FILE = "/etc/piuparts/piuparts.conf"
 
 apt_pkg.init_system()
 
 error_pattern = re.compile(r"(?<=\n).*error.*\n?", flags=re.IGNORECASE)
 chroot_pattern = re.compile(r"tmp/tmp.*?'")
 
+############################################################################
+
+class BTS_Timeout(Exception):
+    pass
+
+
+def alarm_handler(signum, frame):
+    raise BTS_Timeout
+
+
+class PiupartsBTS():
+
+    def __init__(self):
+        self._bugs_usertagged_piuparts = None
+        self._bugs_in_package = dict()
+        self._bugs_affecting_package = dict()
+        self._bug_versions = dict()
+
+        self._queries = 0
+        self._misses = 0
+
+    def all_piuparts_bugs(self):
+        if self._bugs_usertagged_piuparts is None:
+            self._bugs_usertagged_piuparts = debianbts.get_usertag("debian-qa@lists.debian.org", 'piuparts')['piuparts']
+        return self._bugs_usertagged_piuparts
+
+    def bugs_in(self, package):
+        if not package in self._bugs_in_package:
+            self._misses += 1
+            signal(SIGALRM, alarm_handler)
+            alarm(120)
+            bugs = debianbts.get_bugs('package', package, 'bugs', self.all_piuparts_bugs(), 'archive', 'both')
+            bugs += debianbts.get_bugs('package', 'src:' + package, 'bugs', self.all_piuparts_bugs(), 'archive', 'both')
+            alarm(0)
+            self._bugs_in_package[package] = sorted(set(bugs), reverse=True)
+        self._queries += 1
+        return self._bugs_in_package[package]
+
+    def bugs_affecting(self, package):
+        if not package in self._bugs_affecting_package:
+            self._misses += 1
+            signal(SIGALRM, alarm_handler)
+            alarm(120)
+            bugs = debianbts.get_bugs('affects', package, 'bugs', self.all_piuparts_bugs(), 'archive', 'both')
+            bugs += debianbts.get_bugs('affects', 'src:' + package, 'bugs', self.all_piuparts_bugs(), 'archive', 'both')
+            alarm(0)
+            self._bugs_affecting_package[package] = sorted(set(bugs), reverse=True)
+        self._queries += 1
+        return self._bugs_affecting_package[package]
+
+    def bug_versions(self, bug):
+        """Gets a list of only the version numbers for which the bug is found.
+        Newest versions are returned first."""
+        # debianbts returns it in the format package/1.2.3 or 1.2.3 which will become 1.2.3
+        if not bug in self._bug_versions:
+            self._misses += 1
+            signal(SIGALRM, alarm_handler)
+            alarm(60)
+            found_versions = debianbts.get_status(bug)[0].found_versions
+            alarm(0)
+            versions = []
+            for found_version in found_versions:
+                v = found_version.rsplit('/', 1)[-1]
+                if v == "None":
+                    # ignore $DISTRO/None versions
+                    pass
+                else:
+                    versions.append(v)
+            self._bug_versions[bug] =  list(reversed(sorted(versions, cmp=apt_pkg.version_compare))) or ['~']
+        self._queries += 1
+        return self._bug_versions[bug]
+
+    def print_stats(self):
+        print("PiupartsBTS: %d queries, %d forwarded to debianbts" % (self._queries, self._misses))
+
+
+piupartsbts = PiupartsBTS()
+
+############################################################################
+
+class Busy(Exception):
+
+    def __init__(self):
+        self.args = "section is locked by another process",
+
+
+class Config(piupartslib.conf.Config):
+    def __init__(self, section="global", defaults_section=None):
+        self.section = section
+        piupartslib.conf.Config.__init__(self, section,
+                                         {
+                                         "sections": "report",
+                                         "master-directory": ".",
+                                         },
+                                         defaults_section=defaults_section)
+
 
 def find_logs(directory):
     """Returns list of logfiles sorted by age, newest first."""
@@ -81,9 +186,8 @@ def package_source_version(log):
 def extract_errors(log):
     """This pretty stupid implementation is basically just 'grep -i error', and then
     removing the timestamps and the name of the chroot and the package version itself."""
-    f = open(log)
-    data = f.read()
-    f.close()
+    with open(log, "r") as f:
+        data = f.read()
     whole = ''
     pversion = package_version(log)
     for match in error_pattern.finditer(data):
@@ -101,9 +205,8 @@ def extract_errors(log):
 
 
 def extract_headers(log):
-    f = open(log)
-    data = f.read()
-    f.close()
+    with open(log, "r") as f:
+        data = f.read()
     headers = []
     headers = data.partition("\nExecuting:")[0]
     if headers and not headers.endswith("\n"):
@@ -112,13 +215,11 @@ def extract_headers(log):
 
 
 def prepend_to_file(filename, data):
-    f = file(filename, "r")
-    old_data = f.read()
-    f.close()
-    f = file(filename + ".tmp", "w")
-    f.write(data)
-    f.write(old_data)
-    f.close()
+    with open(filename, "r") as f:
+        old_data = f.read()
+    with open(filename + ".tmp", "w") as f:
+        f.write(data)
+        f.write(old_data)
 
     shutil.copymode(filename, filename + ".tmp")
 
@@ -127,27 +228,11 @@ def prepend_to_file(filename, data):
     os.remove(filename + "~")
 
 
-def get_bug_versions(bug):
-    """Gets a list of only the version numbers for which the bug is found.
-    Newest versions are returned first."""
-    # debianbts returns it in the format package/1.2.3 or 1.2.3 which will become 1.2.3
-    versions = []
-    for found_version in debianbts.get_status(bug)[0].found_versions:
-        v = found_version.rsplit('/', 1)[-1]
-        if v == "None":
-            # ignore $DISTRO/None versions
-            pass
-        else:
-            versions.append(v)
-    return list(reversed(sorted(versions, cmp=apt_pkg.version_compare))) or ['~']
-
-
 def write_bug_file(failed_log, bugs):
     if bugs:
-        f = file(os.path.splitext(failed_log)[0] + '.bug', "w")
-        for bug in bugs:
-            f.write('<a href="https://bugs.debian.org/%s" target="_blank">#%s</a>\n' % (bug, bug))
-        f.close()
+        with open(os.path.splitext(failed_log)[0] + '.bug', "w") as f:
+            for bug in bugs:
+                f.write('<a href="https://bugs.debian.org/%s" target="_blank">#%s</a>\n' % (bug, bug))
 
 
 def move_to_bugged(failed_log, bugged="bugged", bug=None):
@@ -184,8 +269,8 @@ def mark_logs_with_reported_bugs():
                 print('IOError while processing %s' % failed_log)
                 continue
             moved = False
-            abugs = piuparts_bugs_affecting(pname)
-            bugs = piuparts_bugs_in(pname)
+            abugs = piupartsbts.bugs_affecting(pname)
+            bugs = piupartsbts.bugs_in(pname)
             for bug in abugs + bugs:
                 if moved:
                     break
@@ -193,7 +278,7 @@ def mark_logs_with_reported_bugs():
                     bugged = "affected"
                 else:
                     bugged = "bugged"
-                found_versions = get_bug_versions(bug)
+                found_versions = piupartsbts.bug_versions(bug)
                 if pversion in found_versions:
                     move_to_bugged(failed_log, bugged, bug)
                     moved = True
@@ -227,32 +312,60 @@ def mark_logs_with_reported_bugs():
         except:
             print('ERROR processing %s' % failed_log)
             print sys.exc_info()[0]
-
-
-piuparts_usertags_cache = None
-
-
-def all_piuparts_bugs():
-    global piuparts_usertags_cache
-    if piuparts_usertags_cache is None:
-        piuparts_usertags_cache = debianbts.get_usertag("debian-qa@lists.debian.org", 'piuparts')['piuparts']
-    return piuparts_usertags_cache
-
-
-def piuparts_bugs_in(package):
-    bugs = debianbts.get_bugs('package', package, 'bugs', all_piuparts_bugs(), 'archive', 'both')
-    bugs += debianbts.get_bugs('package', 'src:' + package, 'bugs', all_piuparts_bugs(), 'archive', 'both')
-    return sorted(set(bugs), reverse=True)
-
-
-def piuparts_bugs_affecting(package):
-    bugs = debianbts.get_bugs('affects', package, 'bugs', all_piuparts_bugs(), 'archive', 'both')
-    bugs += debianbts.get_bugs('affects', 'src:' + package, 'bugs', all_piuparts_bugs(), 'archive', 'both')
-    return sorted(set(bugs), reverse=True)
+        alarm(0)
 
 
 def main():
-    mark_logs_with_reported_bugs()
+    conf = Config()
+    conf.read(CONFIG_FILE)
+
+    master_directory = conf["master-directory"]
+    if len(sys.argv) > 1:
+        sections = sys.argv[1:]
+    else:
+        sections = conf['sections'].split()
+
+    with open(os.path.join(master_directory, "analyze.lock"), "we") as lock:
+        try:
+            fcntl.flock(lock, fcntl.LOCK_EX | fcntl.LOCK_NB)
+        except IOError:
+            if sys.stdout.isatty():
+                sys.exit("another piuparts-analyze process is already running")
+            else:
+                sys.exit(0)
+
+        todo = deque([(s, 0) for s in sections])
+        while len(todo):
+            (section_name, next_try) = todo.popleft()
+            now = time.time()
+            if (now < next_try):
+                print("Sleeping while section is busy")
+                time.sleep(max(30, next_try - now) + 30)
+            print(time.strftime("%a %b %2d %H:%M:%S %Z %Y", time.localtime()))
+            print("%s:" % section_name)
+            try:
+                section_directory = os.path.join(master_directory, section_name)
+                if not os.path.exists(section_directory):
+                    raise MissingSection("", section_name)
+                with open(os.path.join(section_directory, "master.lock"), "we") as lock:
+                    try:
+                        fcntl.flock(lock, fcntl.LOCK_EX | fcntl.LOCK_NB)
+                    except IOError:
+                        raise Busy()
+
+                    oldcwd = os.getcwd()
+                    os.chdir(section_directory)
+                    mark_logs_with_reported_bugs()
+                    os.chdir(oldcwd)
+            except Busy:
+                print("Section is busy")
+                todo.append((section_name, time.time() + 300))
+            except MissingSection as e:
+                print("Configuration Error in section '%s': %s" % (section_name, e))
+            print("")
+
+        print(time.strftime("%a %b %2d %H:%M:%S %Z %Y", time.localtime()))
+        piupartsbts.print_stats()
 
 
 if __name__ == "__main__":
