@@ -15,8 +15,7 @@
 # Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+# this program. If not, see <https://www.gnu.org/licenses/>
 
 
 """Distributed piuparts processing, slave program
@@ -34,7 +33,6 @@ from signal import alarm, signal, SIGALRM, SIGINT, SIGKILL, SIGHUP
 import subprocess
 import fcntl
 import random
-import ConfigParser
 import apt_pkg
 import pipes
 
@@ -261,12 +259,11 @@ class Slave:
         package, rest = basename.split("_", 1)
         version = rest[:-len(".log")]
         self._writeline(pass_or_fail, package, version)
-        f = file(filename, "r")
-        for line in f:
-            if line.endswith("\n"):
-                line = line[:-1]
-            self._writeline(" " + line)
-        f.close()
+        with open(filename, "r") as f:
+            for line in f:
+                if line.endswith("\n"):
+                    line = line[:-1]
+                self._writeline(" " + line)
         self._writeline(".")
         line = self._readline()
         if line != "ok\n":
@@ -672,7 +669,7 @@ class Section:
         output_name = log_name(pname, pvers)
         logging.debug("Opening log file %s" % output_name)
         new_name = os.path.join("new", output_name)
-        output = file(new_name, "we")
+        output = open(new_name, "we")
         output.write(time.strftime("Start: %Y-%m-%d %H:%M:%S %Z\n",
                                    time.gmtime()))
 
@@ -781,11 +778,14 @@ class Section:
             elif not "piuparts run ends" in lastline:
                 ret += 1024
                 output.write(" *** PIUPARTS OUTPUT INCOMPLETE ***\n")
-            elif distupgrade and self._config["chroot-meta-auto"] and \
-                    "History of available packages does not match - reference chroot may be outdated" in f:
+            elif distupgrade and self._config["chroot-meta-auto"]:
                 try:
-                    os.unlink(self._config["chroot-meta-auto"])
-                    logging.info("Deleting outdated %s" % self._config["chroot-meta-auto"])
+                    if "History of available packages does not match - reference chroot may be outdated" in f:
+                        os.unlink(self._config["chroot-meta-auto"])
+                        logging.info("Deleting outdated %s" % self._config["chroot-meta-auto"])
+                    elif "Initial package selections do not match - ignoring loaded reference chroot state" in f:
+                        os.unlink(self._config["chroot-meta-auto"])
+                        logging.info("Deleting mismatching %s" % self._config["chroot-meta-auto"])
                 except OSError:
                     pass
 
@@ -924,9 +924,8 @@ def create_chroot(config, tarball, distro):
 
 
 def create_file(filename, contents):
-    f = file(filename, "w")
-    f.write(contents)
-    f.close()
+    with open(filename, "w") as f:
+        f.write(contents)
 
 
 def main():
