@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Copyright 2005 Lars Wirzenius (liw@iki.fi)
-# Copyright © 2011-2015 Andreas Beckmann (anbe@debian.org)
+# Copyright © 2011-2017 Andreas Beckmann (anbe@debian.org)
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -14,8 +14,7 @@
 # Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+# this program. If not, see <https://www.gnu.org/licenses/>
 
 
 """Packages database for distributed piuparts processing
@@ -70,6 +69,22 @@ class Package(UserDict.UserDict):
         self.waiting_cnt = None
         self.rdep_chain_len = None
 
+    def name(self):
+        return self["Package"]
+
+    def version(self):
+        return self["Version"]
+
+    def set_test_versions(self, tv):
+        self["TestVersions"] = tv
+
+    def test_versions(self):
+        """ToDo: for distupgrade tests test_versions() should return a list
+           of versions in all distros in the upgrade path"""
+        if "TestVersions" in self:
+            return self["TestVersions"]
+        return self.version()
+
     def _parse_dependencies(self, header_name):
         if header_name in self._parsed_deps:
             depends = self._parsed_deps[header_name]
@@ -194,9 +209,6 @@ class LogDB:
             if basename.endswith(".log"):
                 cache[os.path.join(dirname, basename)] = True
 
-    def open_file(self, pathname, mode):
-        return file(pathname, mode)
-
     def remove_file(self, pathname):
         os.remove(pathname)
 
@@ -204,7 +216,7 @@ class LogDB:
         return "%s_%s.log" % (package, version)
 
     def log_exists(self, package, subdirs):
-        log_name = self._log_name(package["Package"], package["Version"])
+        log_name = self._log_name(package.name(), package.test_versions())
         for subdir in subdirs:
             if self.exists(os.path.join(subdir, log_name)):
                 return True
@@ -383,10 +395,10 @@ class PackagesDB:
         pf2 = PackagesFile()
         pf2.load_packages_urls(urls)
         for package in self.get_all_packages():
-            if package["Package"] in pf2:
-                package["Version"] = pf2[package["Package"]]["Version"]
+            if package.name() in pf2:
+                package.set_test_versions(pf2[package.name()].version())
             else:
-                package["Version"] = "None"
+                package.set_test_versions("None")
 
     def get_urls(self):
         urls = []
@@ -450,9 +462,12 @@ class PackagesDB:
     def _lookup_package_state(self, package, use_cached_success, check_outdated):
         if check_outdated:
             # Check if dependency databases have a newer version of this package.
+            # Use the actual package versions, not the target versions.
+            curr_ver = package.version()
             for db in self._dependency_databases:
-                v = db.get_version(package["Package"])
-                if v is not None and apt_pkg.version_compare(package["Version"], v) < 0:
+                dep_ver = db.get_version(package.name())
+                if dep_ver is not None and apt_pkg.version_compare(curr_ver, dep_ver) < 0:
+                    #logging.info("[%s] outdated: %s %s < %s @[%s]" % (self.prefix, package.name(), curr_ver, dep_ver, db.prefix))
                     return "outdated";
         if self._recycle_mode and self._logdb.log_exists(package, [self._recycle]):
             return "unknown"
@@ -661,7 +676,13 @@ class PackagesDB:
     def get_version(self, name):
         self._find_all_packages()
         if name in self._packages:
-            return self._packages[name]["Version"]
+            return self._packages[name].version()
+        return None
+
+    def get_test_versions(self, name):
+        self._find_all_packages()
+        if name in self._packages:
+            return self._packages[name].test_versions()
         return None
 
     def get_source(self, name):
@@ -767,7 +788,7 @@ class PackagesDB:
             )
 
         try:
-            statobj = self._logdb.stat(self._recycle, p["Package"], p["Version"])
+            statobj = self._logdb.stat(self._recycle, p.name(), p.test_versions())
             ctime = statobj[stat.ST_CTIME]  # last inode modification = time of linking into recycle/
             mtime = statobj[stat.ST_MTIME]
         except OSError:
@@ -805,14 +826,14 @@ class PackagesDB:
             if self._recycle_mode and self._logdb.log_exists(p, [self._recycle]):
                 for vdir in all_but_recycle:
                     if self._logdb.log_exists(p, [vdir]):
-                        self._logdb.remove(vdir, p["Package"], p["Version"])
-                        logging.info("Recycled %s %s %s" % (vdir, p["Package"], p["Version"]))
+                        self._logdb.remove(vdir, p.name(), p.test_versions())
+                        logging.info("Recycled %s %s %s" % (vdir, p.name(), p.test_versions()))
             if self._logdb.log_exists(p, all_but_recycle):
                 self._remove_unavailable_candidate(p)
                 continue
             if self._logdb.log_exists(p, [self._recycle]):
-                self._logdb.remove(self._recycle, p["Package"], p["Version"])
-            if self._logdb.create(self._reserved, p["Package"], p["Version"], ""):
+                self._logdb.remove(self._recycle, p.name(), p.test_versions())
+            if self._logdb.create(self._reserved, p.name(), p.test_versions(), ""):
                 return p
         return None
 
